////////////////////////////////////////////////////////////////////////////
//
//  uParse_Demo.c
//
//	Demo-Hauptprogramm
//
//  Hier luft alles zusammen.
//  Es wird lediglich der UART initialisiert, der Parser selbst bentigt 
//  keine Initialisierung.
//  Die Eingaben des UARTs werden behandelt und es sind 3 Beispielfunktionen
//  als Aufruf des Parsers vorhanden. Auerdem gibt es zwei Beispielcodecs
//  (einfaches Datum und Boolean).
//
//	Author 	: Rene Bllhoff
//  Created : around Apr 10
//
//	(c) 2005-2010 by Rene Bllhoff - reneboellhoff (at) gmx (dot) de - 
//
//////////////////////////////////////////////////////////////////////////////
//
//  This source file may be used and distributed without
//  restriction provided that this copyright statement is not
//  removed from the file and that any derivative work contains
//  the original copyright notice and the associated disclaimer.
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//										
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public
//  License along with this source; if not, download it	from
//  http://www.gnu.org/licenses/gpl.html	
//
////////////////////////////////////////////////////////////////////////////		
	
	#include "lib.h"


U8 ucInputPos = 0; // Wird nur gebraucht um ein Delete am Zeilenanfang zu verhindern

void vUARTHandleInput (void)
{
	U8 ucData;
	char *pcErrPos;
	// wenn Zeichen empfangen
  if (bUARTInputReceived)
	{
	  // Flag zurcksetzen
	  ucData = ucUARTReceivedInput;
	  bUARTInputReceived  = FALSE;

		// bei Del (\x08 oder \x7f fr vt100 via putty)
		// letztes Zeichen (sofern wir nicht am Zeilenanfang sind) lschen
		if ((ucData == 0x08) || (ucData == 0x7f))
		{
		  if (ucInputPos > 0)
			{
			  ucInputPos--;
		  	printf_P (PSTR ("\x08 \x08"));
			}
		}
		// ansonsten ausgeben
		else
		{
		  ucInputPos++;
			vUARTSendChar (ucData);
		}
	}

	// Eingabe mit \x0d (Enter) abgeschlossen ?
	if (bUARTProcessInput)
	{
		// CRLF
		printf_P (PSTR ("\n\r"));
		// und Parser aufrufen. Im Fehlerflle bekommen wir mit pucErrPos
		// die Stelle des Fehlers (wenn mehrere Befehle hintereinander stehen)
		if (cParserProcessWErr ((char *)acUARTRecData, &pcErrPos) == 0)
		  printf_P (PSTR ("Ok\n\r"));
	  else
		  printf_P (PSTR ("Err : '%s'\n\r"), pcErrPos);

	  bUARTProcessInput = FALSE;
		ucInputPos = 0;
	}
}

// Parser-Funktion fr "Befehl Aufschlsseln"
void vPrintCmd (U8 ucCmd, T_CMD_DATAS stCmdData)
{
  U8 ucCnt;

	// erstmal sagen "Befehl erkannt"
	printf_P (PSTR ("Befehl %d erkannt!\n\rParameter : "), ucCmd);
	// jedes Element der Parameter durchgehen
  for (ucCnt = 0; ucCnt < PARSE_DATA_ELEMENTS; ucCnt ++)
	{
	  // Komma ausgeben (nur wenn wir nicht beim ersten Element sind)
	  if ((ucCnt != 0) && (stCmdData [ucCnt].ucType != 0)) { printf_P (PSTR (", ")); }
		// Element nach Typen ausgeben
	  switch (stCmdData [ucCnt].ucType)
		{
		  case 1: printf_P (PSTR ("Wert %d"), stCmdData [ucCnt].ulLong);			break;
		  case 2: printf_P (PSTR ("Name %s"), stCmdData [ucCnt].aucText);			break;
		  case 3: printf_P (PSTR ("String '%s'"), stCmdData [ucCnt].aucText);	break;
      // Anzeige von Werten von Eigenen Codecs ab hier ...
		  case 4: printf_P (PSTR ("Bool '%d'"), stCmdData [ucCnt].ulLong);		break;
		  case 5: printf_P (PSTR ("Datum '%04x'"), stCmdData [ucCnt].ulLong);	break;
		}
	}
	printf_P (PSTR ("\n\r"));
}


// Beispiel Funktion fr Parser-Aufruf
void vCmdCtrl (U8 ucCmd, T_CMD_DATAS stCmdDatas)
{
	vUARTSendString_P (PSTR ("Kommando Steuern \n\r"));
	switch (ucCmd)
	{
	  case CMD 			: 	printf_P (PSTR ("Befehl %s ausfhren\n\r"), stCmdDatas [0].aucText);
			break;
	  case STATE		 : 	printf_P (PSTR ("Zustand %d setzen\n\r"), stCmdDatas [0].ulLong);
			break;
	}
  
}

// Beispiel Funktion fr Parser-Aufruf
PARSER_CALL (vCmdSet)   	 // erzeugt void vCmdSet (U8 ucCmd, T_CMD_DATAS stCmdData)
{
  GET_PAR_STR (acName, 0); // erzeugt char *acName = (char *) stCmdDatas [0].aucText;
	GET_PAR_STR (acStr,  1); // erzeugt char *acStr  = (char *) stCmdDatas [1].aucText;
	GET_PAR_I16 (iValue, 1); // erzeugt I16  iValue  = (char *) stCmdDatas [1].ulLong;


	vUARTSendString_P (PSTR ("Wert setzen \n\r"));
	switch (ucCmd)
	{
	  case SETVAL   :
	  case SETVAL2	: 	printf_P (PSTR ("Wert %s zu %d setzen\n\r"), acName, iValue);
			break;
	  case SETSTR   :
	  case SETSTR2	: 	printf_P (PSTR ("Wert %s zu '%s' setzen\n\r"), acName, acStr);
			break;
	}  
}

// Beispiel Funktion fr Parser-Aufruf
void vCmdTest (U8 ucCmd, T_CMD_DATAS stCmdData)
{
  vPrintCmd (ucCmd, stCmdData);
}


// Beispiel Codecs

// Makros ums krzer zu fassen ...
#define ISNUM(x) 		((x >= '0') && (x <= '9'))
#define ADDNUM(x,c)	x *= 10;x += c -'0';

// Beispiel eines Codecs mittels einer Statemachine
char vCodecDate (char **ppucText, T_CMD_DATA *pstData)
{
	char cSt = 1, cCh, cDay = 0, cMon = 0, cYear = 0;
	do 
	{
		cCh = **ppucText; (*ppucText)++;
		switch (cSt)
		{
			case 1 :	if (ISNUM (cCh))	{ cSt =  2; ADDNUM (cDay, cCh); }	else	{ cSt = -1; }	break;
			case 2 :	if (ISNUM (cCh))	{ cSt =  3; ADDNUM (cDay, cCh); }	else	
			          if ((cCh == '.'))                  	 { cSt =  4;  }	else  { cSt = -1; }	break;
			case 3 :  if ((cCh == '.'))                  	 { cSt =  4;  }	else  { cSt = -1; }	break;

			case 4 :	if (ISNUM (cCh))	{ cSt =  5; ADDNUM (cMon, cCh); }	else	{ cSt = -1; }	break;
			case 5 :	if (ISNUM (cCh))	{ cSt =  6; ADDNUM (cMon, cCh); }	else	
			          if ((cCh == '.'))                  	 { cSt =  7;  }	else  { cSt = -1; }	break;
			case 6 :  if ((cCh == '.'))                  	 { cSt =  7;  }	else  { cSt = -1; }	break;

			case 7 :	if (ISNUM (cCh))	{ cSt =  8; ADDNUM (cYear, cCh);}	else	{ cSt = -1; }	break;
			case 8 :	if (ISNUM (cCh))	{ cSt =  0; ADDNUM (cYear, cCh);}	else	{ cSt = -1; }	break;
		}
	} while (cSt > 0);
	if (cSt == 0) 
	{ 
		pstData->ulLong = (((U32) cDay) << 0) | (((U32) cMon) << 5) | (((U32) cYear) << 9); 
	}
	return cSt;
}

// Beispiel eines Codecs mittels eines einfachen Stringvergleichs
// Wichtig ist den Zeiger korrekt zurckzugeben, und
// strncmp zu verwenden, da dahinter noch relevanter Text 
// stehen kann !!
char vCodecBool (char **ppucText, T_CMD_DATA *pstData)
{
	char 	cLen = 0, // lnge des erkannten strings (!= 0) oder
	                // string nicht erkannt (== 0)
				*pcText = *ppucText;  // temporrer Zeiger um einfacher arbeiten zu knnen
	
	// vergleichen, wert und lnge neu setzen
	if (strncmp (pcText, "true", 4) == 0) 	{ pstData->ulLong = 1; cLen = 4; }
	if (strncmp (pcText, "false", 5) == 0) 	{ pstData->ulLong = 0; cLen = 5; }
	if (strncmp (pcText, "1", 1) == 0)    	{ pstData->ulLong = 1; cLen = 1; }
	if (strncmp (pcText, "0", 1) == 0)     	{ pstData->ulLong = 0; cLen = 1; }
	if (strncmp (pcText, "yes", 3) == 0) 		{ pstData->ulLong = 1; cLen = 3; }
	if (strncmp (pcText, "no", 2) == 0) 		{ pstData->ulLong = 0; cLen = 2; }
	if (strncmp (pcText, "on", 2) == 0) 		{ pstData->ulLong = 1; cLen = 2; }
	if (strncmp (pcText, "off", 3) == 0) 		{ pstData->ulLong = 0; cLen = 3; }

	pcText = &pcText [(U8) cLen]; // im Fehlerfall wird Zeiger "nicht" bewegt
	*ppucText = pcText;

	return cLen == 0;
}


int main (void)
{
	vUARTInit ();
	sei ();

	// vt100 clear screen
	printf_P (PSTR ("\x1b[0m\x1b[2J\x1b[0;0H"));
	// startup prompt
	printf_P (PSTR ("Parser Demo\n\rEnter 'help' for help\n\r"));
	printf_P (PSTR ("Ok\n\r"));


	while (1)
	{
	  vUARTHandleInput ();
	}

}
